@@ -2,6 +2,7 @@
 
 import path from "path";
 import buildDebug from "debug";
+import type { Handler } from "gensync";
 import {
   validate,
   type ValidatedOptions,
@@ -11,6 +12,7 @@ import {
   type CallerMetadata,
 } from "./validation/options";
 import pathPatternToRegex from "./pattern-to-regex";
+import { ConfigPrinter, ChainFormatter } from "./printer";
 
 const debug = buildDebug("babel:config:config-chain");
 
@@ -24,7 +26,7 @@ import {
   type FilePackageData,
 } from "./files";
 
-import { makeWeakCache, makeStrongCache } from "./caching";
+import { makeWeakCacheSync, makeStrongCacheSync } from "./caching";
 
 import {
   createCachedDescriptors,
@@ -52,16 +54,17 @@ export type ConfigContext = {
   root: string,
   envName: string,
   caller: CallerMetadata | void,
+  showConfig: boolean,
 };
 
 /**
  * Build a config chain for a given preset.
  */
-export function buildPresetChain(
+export function* buildPresetChain(
   arg: PresetInstance,
   context: *,
-): ConfigChain | null {
-  const chain = buildPresetChainWalker(arg, context);
+): Handler<ConfigChain | null> {
+  const chain = yield* buildPresetChainWalker(arg, context);
   if (!chain) return null;
 
   return {
@@ -75,18 +78,18 @@ export const buildPresetChainWalker: (
   arg: PresetInstance,
   context: *,
 ) => * = makeChainWalker({
-  init: arg => arg,
   root: preset => loadPresetDescriptors(preset),
   env: (preset, envName) => loadPresetEnvDescriptors(preset)(envName),
   overrides: (preset, index) => loadPresetOverridesDescriptors(preset)(index),
   overridesEnv: (preset, index, envName) =>
     loadPresetOverridesEnvDescriptors(preset)(index)(envName),
+  createLogger: () => () => {}, // Currently we don't support logging how preset is expanded
 });
-const loadPresetDescriptors = makeWeakCache((preset: PresetInstance) =>
+const loadPresetDescriptors = makeWeakCacheSync((preset: PresetInstance) =>
   buildRootDescriptors(preset, preset.alias, createUncachedDescriptors),
 );
-const loadPresetEnvDescriptors = makeWeakCache((preset: PresetInstance) =>
-  makeStrongCache((envName: string) =>
+const loadPresetEnvDescriptors = makeWeakCacheSync((preset: PresetInstance) =>
+  makeStrongCacheSync((envName: string) =>
     buildEnvDescriptors(
       preset,
       preset.alias,
@@ -95,20 +98,21 @@ const loadPresetEnvDescriptors = makeWeakCache((preset: PresetInstance) =>
     ),
   ),
 );
-const loadPresetOverridesDescriptors = makeWeakCache((preset: PresetInstance) =>
-  makeStrongCache((index: number) =>
-    buildOverrideDescriptors(
-      preset,
-      preset.alias,
-      createUncachedDescriptors,
-      index,
-    ),
-  ),
-);
-const loadPresetOverridesEnvDescriptors = makeWeakCache(
+const loadPresetOverridesDescriptors = makeWeakCacheSync(
   (preset: PresetInstance) =>
-    makeStrongCache((index: number) =>
-      makeStrongCache((envName: string) =>
+    makeStrongCacheSync((index: number) =>
+      buildOverrideDescriptors(
+        preset,
+        preset.alias,
+        createUncachedDescriptors,
+        index,
+      ),
+    ),
+);
+const loadPresetOverridesEnvDescriptors = makeWeakCacheSync(
+  (preset: PresetInstance) =>
+    makeStrongCacheSync((index: number) =>
+      makeStrongCacheSync((envName: string) =>
         buildOverrideEnvDescriptors(
           preset,
           preset.alias,
@@ -129,39 +133,55 @@ export type RootConfigChain = ConfigChain & {
 /**
  * Build a config chain for Babel's full root configuration.
  */
-export function buildRootChain(
+export function* buildRootChain(
   opts: ValidatedOptions,
   context: ConfigContext,
-): RootConfigChain | null {
-  const programmaticChain = loadProgrammaticChain(
+): Handler<RootConfigChain | null> {
+  let configReport, babelRcReport;
+  const programmaticLogger = new ConfigPrinter();
+  const programmaticChain = yield* loadProgrammaticChain(
     {
       options: opts,
       dirname: context.cwd,
     },
     context,
+    undefined,
+    programmaticLogger,
   );
   if (!programmaticChain) return null;
+  const programmaticReport = programmaticLogger.output();
 
   let configFile;
   if (typeof opts.configFile === "string") {
-    configFile = loadConfig(
+    configFile = yield* loadConfig(
       opts.configFile,
       context.cwd,
       context.envName,
       context.caller,
     );
   } else if (opts.configFile !== false) {
-    configFile = findRootConfig(context.root, context.envName, context.caller);
+    configFile = yield* findRootConfig(
+      context.root,
+      context.envName,
+      context.caller,
+    );
   }
 
   let { babelrc, babelrcRoots } = opts;
   let babelrcRootsDirectory = context.cwd;
 
   const configFileChain = emptyChain();
+  const configFileLogger = new ConfigPrinter();
   if (configFile) {
     const validatedFile = validateConfigFile(configFile);
-    const result = loadFileChain(validatedFile, context);
+    const result = yield* loadFileChain(
+      validatedFile,
+      context,
+      undefined,
+      configFileLogger,
+    );
     if (!result) return null;
+    configReport = configFileLogger.output();
 
     // Allow config files to toggle `.babelrc` resolution on and off and
     // specify where the roots are.
@@ -178,7 +198,7 @@ export function buildRootChain(
 
   const pkgData =
     typeof context.filename === "string"
-      ? findPackageData(context.filename)
+      ? yield* findPackageData(context.filename)
       : null;
 
   let ignoreFile, babelrcFile;
@@ -189,7 +209,7 @@ export function buildRootChain(
     pkgData &&
     babelrcLoadEnabled(context, pkgData, babelrcRoots, babelrcRootsDirectory)
   ) {
-    ({ ignore: ignoreFile, config: babelrcFile } = findRelativeConfig(
+    ({ ignore: ignoreFile, config: babelrcFile } = yield* findRelativeConfig(
       pkgData,
       context.envName,
       context.caller,
@@ -203,13 +223,32 @@ export function buildRootChain(
     }
 
     if (babelrcFile) {
-      const result = loadFileChain(validateBabelrcFile(babelrcFile), context);
+      const validatedFile = validateBabelrcFile(babelrcFile);
+      const babelrcLogger = new ConfigPrinter();
+      const result = yield* loadFileChain(
+        validatedFile,
+        context,
+        undefined,
+        babelrcLogger,
+      );
       if (!result) return null;
+      babelRcReport = babelrcLogger.output();
 
       mergeChain(fileChain, result);
     }
   }
 
+  if (context.showConfig) {
+    console.log(
+      // $FlowIgnore: context.showConfig implies context.filename is not null
+      `Babel configs on "${context.filename}" (ascending priority):\n` +
+        // print config by the order of ascending priority
+        [configReport, babelRcReport, programmaticReport]
+          .filter(x => !!x)
+          .join("\n\n"),
+    );
+    return null;
+  }
   // Insert file chain in front so programmatic options have priority
   // over configuration file chain items.
   const chain = mergeChain(
@@ -268,7 +307,7 @@ function babelrcLoadEnabled(
   });
 }
 
-const validateConfigFile = makeWeakCache(
+const validateConfigFile = makeWeakCacheSync(
   (file: ConfigFile): ValidatedFile => ({
     filepath: file.filepath,
     dirname: file.dirname,
@@ -276,7 +315,7 @@ const validateConfigFile = makeWeakCache(
   }),
 );
 
-const validateBabelrcFile = makeWeakCache(
+const validateBabelrcFile = makeWeakCacheSync(
   (file: ConfigFile): ValidatedFile => ({
     filepath: file.filepath,
     dirname: file.dirname,
@@ -284,7 +323,7 @@ const validateBabelrcFile = makeWeakCache(
   }),
 );
 
-const validateExtendFile = makeWeakCache(
+const validateExtendFile = makeWeakCacheSync(
   (file: ConfigFile): ValidatedFile => ({
     filepath: file.filepath,
     dirname: file.dirname,
@@ -309,6 +348,8 @@ const loadProgrammaticChain = makeChainWalker({
       index,
       envName,
     ),
+  createLogger: (input, context, baseLogger) =>
+    buildProgrammaticLogger(input, context, baseLogger),
 });
 
 /**
@@ -320,12 +361,14 @@ const loadFileChain = makeChainWalker({
   overrides: (file, index) => loadFileOverridesDescriptors(file)(index),
   overridesEnv: (file, index, envName) =>
     loadFileOverridesEnvDescriptors(file)(index)(envName),
+  createLogger: (file, context, baseLogger) =>
+    buildFileLogger(file.filepath, context, baseLogger),
 });
-const loadFileDescriptors = makeWeakCache((file: ValidatedFile) =>
+const loadFileDescriptors = makeWeakCacheSync((file: ValidatedFile) =>
   buildRootDescriptors(file, file.filepath, createUncachedDescriptors),
 );
-const loadFileEnvDescriptors = makeWeakCache((file: ValidatedFile) =>
-  makeStrongCache((envName: string) =>
+const loadFileEnvDescriptors = makeWeakCacheSync((file: ValidatedFile) =>
+  makeStrongCacheSync((envName: string) =>
     buildEnvDescriptors(
       file,
       file.filepath,
@@ -334,8 +377,8 @@ const loadFileEnvDescriptors = makeWeakCache((file: ValidatedFile) =>
     ),
   ),
 );
-const loadFileOverridesDescriptors = makeWeakCache((file: ValidatedFile) =>
-  makeStrongCache((index: number) =>
+const loadFileOverridesDescriptors = makeWeakCacheSync((file: ValidatedFile) =>
+  makeStrongCacheSync((index: number) =>
     buildOverrideDescriptors(
       file,
       file.filepath,
@@ -344,24 +387,47 @@ const loadFileOverridesDescriptors = makeWeakCache((file: ValidatedFile) =>
     ),
   ),
 );
-const loadFileOverridesEnvDescriptors = makeWeakCache((file: ValidatedFile) =>
-  makeStrongCache((index: number) =>
-    makeStrongCache((envName: string) =>
-      buildOverrideEnvDescriptors(
-        file,
-        file.filepath,
-        createUncachedDescriptors,
-        index,
-        envName,
+const loadFileOverridesEnvDescriptors = makeWeakCacheSync(
+  (file: ValidatedFile) =>
+    makeStrongCacheSync((index: number) =>
+      makeStrongCacheSync((envName: string) =>
+        buildOverrideEnvDescriptors(
+          file,
+          file.filepath,
+          createUncachedDescriptors,
+          index,
+          envName,
+        ),
       ),
     ),
-  ),
 );
 
+function buildFileLogger(
+  filepath: string,
+  context: ConfigContext,
+  baseLogger: ConfigPrinter | void,
+) {
+  if (!baseLogger) {
+    return () => {};
+  }
+  return baseLogger.configure(context.showConfig, ChainFormatter.Config, {
+    filepath,
+  });
+}
+
 function buildRootDescriptors({ dirname, options }, alias, descriptors) {
   return descriptors(dirname, options, alias);
 }
 
+function buildProgrammaticLogger(_, context, baseLogger: ConfigPrinter | void) {
+  if (!baseLogger) {
+    return () => {};
+  }
+  return baseLogger.configure(context.showConfig, ChainFormatter.Programmatic, {
+    callerName: context.caller?.name,
+  });
+}
+
 function buildEnvDescriptors(
   { dirname, options },
   alias,
@@ -409,37 +475,68 @@ function makeChainWalker<ArgT: { options: ValidatedOptions, dirname: string }>({
   env,
   overrides,
   overridesEnv,
-}: {
+  createLogger,
+}: {|
   root: ArgT => OptionsAndDescriptors,
   env: (ArgT, string) => OptionsAndDescriptors | null,
   overrides: (ArgT, number) => OptionsAndDescriptors,
   overridesEnv: (ArgT, number, string) => OptionsAndDescriptors | null,
-}): (ArgT, ConfigContext, Set<ConfigFile> | void) => ConfigChain | null {
-  return (input, context, files = new Set()) => {
+  createLogger: (
+    ArgT,
+    ConfigContext,
+    ConfigPrinter | void,
+  ) => (OptionsAndDescriptors, ?number, ?string) => void,
+|}): (
+  ArgT,
+  ConfigContext,
+  files?: Set<ConfigFile> | void,
+  baseLogger: ConfigPrinter | void,
+) => Handler<ConfigChain | null> {
+  return function* (input, context, files = new Set(), baseLogger) {
     const { dirname } = input;
 
-    const flattenedConfigs = [];
+    const flattenedConfigs: Array<{|
+      config: OptionsAndDescriptors,
+      index: ?number,
+      envName: ?string,
+    |}> = [];
 
     const rootOpts = root(input);
     if (configIsApplicable(rootOpts, dirname, context)) {
-      flattenedConfigs.push(rootOpts);
+      flattenedConfigs.push({
+        config: rootOpts,
+        envName: undefined,
+        index: undefined,
+      });
 
       const envOpts = env(input, context.envName);
       if (envOpts && configIsApplicable(envOpts, dirname, context)) {
-        flattenedConfigs.push(envOpts);
+        flattenedConfigs.push({
+          config: envOpts,
+          envName: context.envName,
+          index: undefined,
+        });
       }
 
       (rootOpts.options.overrides || []).forEach((_, index) => {
         const overrideOps = overrides(input, index);
         if (configIsApplicable(overrideOps, dirname, context)) {
-          flattenedConfigs.push(overrideOps);
+          flattenedConfigs.push({
+            config: overrideOps,
+            index,
+            envName: undefined,
+          });
 
           const overrideEnvOpts = overridesEnv(input, index, context.envName);
           if (
             overrideEnvOpts &&
             configIsApplicable(overrideEnvOpts, dirname, context)
           ) {
-            flattenedConfigs.push(overrideEnvOpts);
+            flattenedConfigs.push({
+              config: overrideEnvOpts,
+              index,
+              envName: context.envName,
+            });
           }
         }
       });
@@ -449,7 +546,7 @@ function makeChainWalker<ArgT: { options: ValidatedOptions, dirname: string }>({
     // that we don't do extra work loading extended configs if a file is
     // ignored.
     if (
-      flattenedConfigs.some(({ options: { ignore, only } }) =>
+      flattenedConfigs.some(({ config: { options: { ignore, only } } }) =>
         shouldIgnore(context, ignore, only, dirname),
       )
     ) {
@@ -457,28 +554,40 @@ function makeChainWalker<ArgT: { options: ValidatedOptions, dirname: string }>({
     }
 
     const chain = emptyChain();
+    const logger = createLogger(input, context, baseLogger);
 
-    for (const op of flattenedConfigs) {
-      if (!mergeExtendsChain(chain, op.options, dirname, context, files)) {
+    for (const { config, index, envName } of flattenedConfigs) {
+      if (
+        !(yield* mergeExtendsChain(
+          chain,
+          config.options,
+          dirname,
+          context,
+          files,
+          baseLogger,
+        ))
+      ) {
         return null;
       }
 
-      mergeChainOpts(chain, op);
+      logger(config, index, envName);
+      mergeChainOpts(chain, config);
     }
     return chain;
   };
 }
 
-function mergeExtendsChain(
+function* mergeExtendsChain(
   chain: ConfigChain,
   opts: ValidatedOptions,
   dirname: string,
   context: ConfigContext,
   files: Set<ConfigFile>,
-): boolean {
+  baseLogger: ConfigPrinter | void,
+): Handler<boolean> {
   if (opts.extends === undefined) return true;
 
-  const file = loadConfig(
+  const file = yield* loadConfig(
     opts.extends,
     dirname,
     context.envName,
@@ -494,7 +603,12 @@ function mergeExtendsChain(
   }
 
   files.add(file);
-  const fileChain = loadFileChain(validateExtendFile(file), context, files);
+  const fileChain = yield* loadFileChain(
+    validateExtendFile(file),
+    context,
+    files,
+    baseLogger,
+  );
   files.delete(file);
 
   if (!fileChain) return false;
@@ -549,7 +663,7 @@ function normalizeOptions(opts: ValidatedOptions): ValidatedOptions {
 
   // "sourceMap" is just aliased to sourceMap, so copy it over as
   // we merge the options together.
-  if (options.hasOwnProperty("sourceMap")) {
+  if (Object.prototype.hasOwnProperty.call(options, "sourceMap")) {
     options.sourceMaps = options.sourceMap;
     delete options.sourceMap;
   }
@@ -631,22 +745,28 @@ function shouldIgnore(
   dirname: string,
 ): boolean {
   if (ignore && matchesPatterns(context, ignore, dirname)) {
-    debug(
-      "Ignored %o because it matched one of %O from %o",
-      context.filename,
+    const message = `No config is applied to "${
+      context.filename ?? "(unknown)"
+    }" because it matches one of \`ignore: ${JSON.stringify(
       ignore,
-      dirname,
-    );
+    )}\` from "${dirname}"`;
+    debug(message);
+    if (context.showConfig) {
+      console.log(message);
+    }
     return true;
   }
 
   if (only && !matchesPatterns(context, only, dirname)) {
-    debug(
-      "Ignored %o because it failed to match one of %O from %o",
-      context.filename,
+    const message = `No config is applied to "${
+      context.filename ?? "(unknown)"
+    }" because it fails to match one of \`only: ${JSON.stringify(
       only,
-      dirname,
-    );
+    )}\` from "${dirname}"`;
+    debug(message);
+    if (context.showConfig) {
+      console.log(message);
+    }
     return true;
   }
 
