@@ -1,6 +1,8 @@
 import { declare } from "@babel/helper-plugin-utils";
 import hoistVariables from "@babel/helper-hoist-variables";
 import { template, types as t } from "@babel/core";
+import { getImportSource } from "babel-plugin-dynamic-import-node/utils";
+import { rewriteThis, getModuleName } from "@babel/helper-module-transforms";
 
 const buildTemplate = template(`
   SYSTEM_REGISTER(MODULE_NAME, SOURCES, function (EXPORT_IDENTIFIER, CONTEXT_IDENTIFIER) {
@@ -21,6 +23,12 @@ const buildExportAll = template(`
   }
 `);
 
+const MISSING_PLUGIN_WARNING = `\
+WARNING: Dynamic import() transformation must be enabled using the
+         @babel/plugin-proposal-dynamic-import plugin. Babel 8 will
+         no longer transform import() without using that plugin.
+`;
+
 function constructExportCall(
   path,
   exportIdent,
@@ -96,12 +104,10 @@ function constructExportCall(
   return statements;
 }
 
-const TYPE_IMPORT = "Import";
-
 export default declare((api, options) => {
   api.assertVersion(7);
 
-  const { systemGlobal = "System" } = options;
+  const { systemGlobal = "System", allowTopLevelThis = false } = options;
   const IGNORE_REASSIGNMENT_SYMBOL = Symbol();
 
   const reassignmentVisitor = {
@@ -168,16 +174,24 @@ export default declare((api, options) => {
   return {
     name: "transform-modules-systemjs",
 
+    pre() {
+      this.file.set("@babel/plugin-transform-modules-*", "systemjs");
+    },
+
     visitor: {
       CallExpression(path, state) {
-        if (path.node.callee.type === TYPE_IMPORT) {
+        if (t.isImport(path.node.callee)) {
+          if (!this.file.has("@babel/plugin-proposal-dynamic-import")) {
+            console.warn(MISSING_PLUGIN_WARNING);
+          }
+
           path.replaceWith(
             t.callExpression(
               t.memberExpression(
                 t.identifier(state.contextIdent),
                 t.identifier("import"),
               ),
-              path.node.arguments,
+              [getImportSource(t, path.node)],
             ),
           );
         }
@@ -214,10 +228,13 @@ export default declare((api, options) => {
       Program: {
         enter(path, state) {
           state.contextIdent = path.scope.generateUid("context");
+          if (!allowTopLevelThis) {
+            rewriteThis(path);
+          }
         },
         exit(path, state) {
-          const undefinedIdent = path.scope.buildUndefinedNode();
-          const exportIdent = path.scope.generateUid("export");
+          const scope = path.scope;
+          const exportIdent = scope.generateUid("export");
           const contextIdent = state.contextIdent;
 
           const exportMap = Object.create(null);
@@ -236,7 +253,7 @@ export default declare((api, options) => {
 
           function pushModule(source, key, specifiers) {
             let module;
-            modules.forEach(function(m) {
+            modules.forEach(function (m) {
               if (m.key === source) {
                 module = m;
               }
@@ -268,7 +285,7 @@ export default declare((api, options) => {
               beforeBody.push(path.node);
               removedPaths.push(path);
             } else if (path.isClassDeclaration()) {
-              variableIds.push(path.node.id);
+              variableIds.push(t.cloneNode(path.node.id));
               path.replaceWith(
                 t.expressionStatement(
                   t.assignmentExpression(
@@ -282,7 +299,7 @@ export default declare((api, options) => {
               const source = path.node.source.value;
               pushModule(source, "imports", path.node.specifiers);
               for (const name of Object.keys(path.getBindingIdentifiers())) {
-                path.scope.removeBinding(name);
+                scope.removeBinding(name);
                 variableIds.push(t.identifier(name));
               }
               path.remove();
@@ -295,8 +312,8 @@ export default declare((api, options) => {
               if (declar.isClassDeclaration()) {
                 if (id) {
                   exportNames.push("default");
-                  exportValues.push(undefinedIdent);
-                  variableIds.push(id);
+                  exportValues.push(scope.buildUndefinedNode());
+                  variableIds.push(t.cloneNode(id));
                   addExportName(id.name, "default");
                   path.replaceWith(
                     t.expressionStatement(
@@ -343,8 +360,8 @@ export default declare((api, options) => {
                 } else if (path.isClass()) {
                   const name = declar.node.id.name;
                   exportNames.push(name);
-                  exportValues.push(undefinedIdent);
-                  variableIds.push(declar.node.id);
+                  exportValues.push(scope.buildUndefinedNode());
+                  variableIds.push(t.cloneNode(declar.node.id));
                   path.replaceWith(
                     t.expressionStatement(
                       t.assignmentExpression(
@@ -364,7 +381,7 @@ export default declare((api, options) => {
                 }
               } else {
                 const specifiers = path.node.specifiers;
-                if (specifiers && specifiers.length) {
+                if (specifiers?.length) {
                   if (path.node.source) {
                     pushModule(path.node.source.value, "exports", specifiers);
                     path.remove();
@@ -372,9 +389,7 @@ export default declare((api, options) => {
                     const nodes = [];
 
                     for (const specifier of specifiers) {
-                      const binding = path.scope.getBinding(
-                        specifier.local.name,
-                      );
+                      const binding = scope.getBinding(specifier.local.name);
                       // hoisted function export
                       if (
                         binding &&
@@ -400,14 +415,16 @@ export default declare((api, options) => {
 
                     path.replaceWithMultiple(nodes);
                   }
+                } else {
+                  path.remove();
                 }
               }
             }
           }
 
-          modules.forEach(function(specifiers) {
+          modules.forEach(function (specifiers) {
             let setterBody = [];
-            const target = path.scope.generateUid(specifiers.key);
+            const target = scope.generateUid(specifiers.key);
 
             for (let specifier of specifiers.imports) {
               if (t.isImportNamespaceSpecifier(specifier)) {
@@ -482,7 +499,7 @@ export default declare((api, options) => {
             );
           });
 
-          let moduleName = this.getModuleName();
+          let moduleName = getModuleName(this.file.opts, options);
           if (moduleName) moduleName = t.stringLiteral(moduleName);
 
           hoistVariables(
@@ -491,7 +508,7 @@ export default declare((api, options) => {
               variableIds.push(id);
               if (!hasInit) {
                 exportNames.push(name);
-                exportValues.push(undefinedIdent);
+                exportValues.push(scope.buildUndefinedNode());
               }
             },
             null,
@@ -521,7 +538,7 @@ export default declare((api, options) => {
           path.traverse(reassignmentVisitor, {
             exports: exportMap,
             buildCall: buildExportCall,
-            scope: path.scope,
+            scope,
           });
 
           for (const path of removedPaths) {
