@@ -9,6 +9,8 @@ export default (_: any, opts: Object = {}) => {
     decoratorsLegacy = false,
     decoratorsBeforeExport,
     pipelineProposal = "minimal",
+    recordAndTupleSyntax: recordAndTupleSyntax = "hash",
+    moduleAttributesVersion = "may-2020",
   } = opts;
 
   return {
@@ -19,11 +21,15 @@ export default (_: any, opts: Object = {}) => {
       ],
     ],
     plugins: [
+      babelPlugins.syntaxDecimal,
+      [
+        babelPlugins.syntaxModuleAttributes,
+        { version: moduleAttributesVersion },
+      ],
+      [babelPlugins.syntaxRecordAndTuple, { syntaxType: recordAndTupleSyntax }],
       babelPlugins.proposalExportDefaultFrom,
-      babelPlugins.proposalLogicalAssignmentOperators,
-      [babelPlugins.proposalOptionalChaining, { loose }],
       [babelPlugins.proposalPipelineOperator, { proposal: pipelineProposal }],
-      [babelPlugins.proposalNullishCoalescingOperator, { loose }],
+      babelPlugins.proposalPrivatePropertyInObject,
       babelPlugins.proposalDoExpressions,
     ],
   };
