@@ -1,7 +1,6 @@
 // @flow
 import toFastProperties from "to-fast-properties";
 import "./core";
-import "./es2015";
 import "./flow";
 import "./jsx";
 import "./misc";
@@ -14,6 +13,7 @@ import {
   NODE_FIELDS,
   BUILDER_KEYS,
   DEPRECATED_KEYS,
+  NODE_PARENT_VALIDATIONS,
 } from "./utils";
 import {
   PLACEHOLDERS,
@@ -43,6 +43,7 @@ export {
   NODE_FIELDS,
   BUILDER_KEYS,
   DEPRECATED_KEYS,
+  NODE_PARENT_VALIDATIONS,
   PLACEHOLDERS,
   PLACEHOLDERS_ALIAS,
   PLACEHOLDERS_FLIPPED_ALIAS,
