@@ -4,6 +4,7 @@ import {
   isCatchClause,
   isBlockStatement,
   isScopable,
+  isPattern,
 } from "./generated";
 
 /**
@@ -18,5 +19,11 @@ export default function isScope(node: Object, parent: Object): boolean {
     return false;
   }
 
+  // If a Pattern is an immediate descendent of a Function, it must be in the params.
+  // Hence we skipped the parentKey === "params" check
+  if (isPattern(node) && isFunction(parent)) {
+    return true;
+  }
+
   return isScopable(node);
 }
